package com.xydl.cac.iec;

import com.beanit.iec61850bean.*;
import com.xydl.cac.entity.constants.Constants;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.io.IOUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.util.CollectionUtils;

import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.net.InetAddress;
import java.nio.charset.StandardCharsets;
import java.util.List;

@Slf4j
public class IecClient implements ClientEventListener {

    ClientSap clientSap = new ClientSap();
    ClientAssociation clientAssociation = null;
    ServerModel serverModel;

    public void connect(String host, int port, String apTitle, String xml) throws Exception {
        InputStream in = IOUtils.toInputStream(xml, StandardCharsets.UTF_8);
        this.connect(host, port, apTitle, in);
    }

    public void connect(String host, int port, String apTitle, InputStream in) throws Exception {
        int[] title = new int[]{1, 3, 9999, 33};
        if (StringUtils.isNotBlank(apTitle)) {
            String[] strs = apTitle.replaceAll(" ", ",").split(",");
            if (strs.length > 3) {
                title[0] = Integer.parseInt(strs[0]);
                title[1] = Integer.parseInt(strs[1]);
                title[2] = Integer.parseInt(strs[2]);
                title[3] = Integer.parseInt(strs[3]);
            }
        }
        clientSap.setTSelRemote(new byte[]{0, 1});
        clientSap.setTSelLocal(new byte[]{0, 0});
        clientSap.setApTitleCalled(title);

        clientAssociation = clientSap.associate(InetAddress.getByName(host), port, null, this);
        serverModel = SclParser.parse(in).get(0);
        clientAssociation.setServerModel(serverModel);
    }

    public void disconnect() {
        try {
            clientAssociation.disconnect();
        } catch (Exception ignore) {
        }
    }

    public String getValue(String paramindex, String fc) throws Exception {
        FcModelNode node = (FcModelNode) serverModel.findModelNode(paramindex, Fc.valueOf(fc));
        clientAssociation.getDataValues(node);
        if (node instanceof BasicDataAttribute) {
            String value = ((BasicDataAttribute) node).getValueString();
            return value;
        }
        return null;
    }

    public void enableReporting() throws Exception {
        if (!CollectionUtils.isEmpty(serverModel.getUrcbs())) {
            for (Urcb urcb : serverModel.getUrcbs()) {
                clientAssociation.enableReporting(urcb);
            }
        }
        if (!CollectionUtils.isEmpty(serverModel.getBrcbs())) {
            for (Brcb brcb : serverModel.getBrcbs()) {
                clientAssociation.enableReporting(brcb);
            }
        }
    }

    public void disableReporting() {
        if (!CollectionUtils.isEmpty(serverModel.getUrcbs())) {
            try {
                for (Urcb urcb : serverModel.getUrcbs()) {
                    clientAssociation.disableReporting(urcb);
                }
            } catch (Exception ignore) {
            }
        }
        if (!CollectionUtils.isEmpty(serverModel.getBrcbs())) {
            try {
                for (Brcb brcb : serverModel.getBrcbs()) {
                    clientAssociation.disableReporting(brcb);
                }
            } catch (Exception ignore) {
            }
        }
    }

    public List<FileInformation> listFile(String path) throws Exception {
        List<FileInformation> list = clientAssociation.getFileDirectory(path);
        return list;
    }

    public void getFile(String remotePath, String filename, String localPath, Integer todel) throws Exception {
        GetFileAction gfa = new GetFileAction(localPath);
        clientAssociation.getFile(remotePath + filename, gfa);
        if (todel != null && todel == Constants.TRUE) {
            clientAssociation.deleteFile(remotePath + filename);
        }
    }

    @Override
    public void newReport(Report report) {
        if (report != null) {
            RealTimeDataService.processReport(report);
        }
    }

    @Override
    public void associationClosed(IOException e) {

    }

    public static void main(String[] args) {
        IecClient iecClient = new IecClient();
        try {
            iecClient.connect("192.168.1.17", 102, "1 3 9999 33", new FileInputStream("C:/资料/om.SCD"));
            String str = iecClient.getValue("OMDLMONT/SPDC1.MaxDsch.mag.f", "MX");
            System.out.println(str);
            str = iecClient.getValue("OMDLMONT/SPDC1.MaxDsch.t", "MX");
            System.out.println(str);
            List<FileInformation> list = iecClient.listFile("/fdata");
            iecClient.getFile("/fdata/web/comtrade/1/18/6/", "20241021_113736_703_0_0.dat", "/eqmid/20241021_113736_703_0_0.dat", 1);

        } catch (Exception ex) {
            ex.printStackTrace();
        } finally {
            iecClient.disconnect();
        }
    }
}
